%HW2, PHYS 3317
%tunneling through a barrier

hbar=1.054571596;              %Planck's constant (x10^34 J s)
echarge=1.602176462;		   %electron charge (x10^19 C)
emass=9.10938188;              %electron mass (x10^31 kg)

% prepare potential
len = 45;	                     %length of well (nm)
npoints=450;					 %number of sample points
x=linspace(0,len,npoints+1)';	 %position of sample points
dx = x(2)-x(1);
num_sol=180;                      %number of solutions sought

v = zeros(size(x));              %potential (eV)
v(x >= 20 & x <= 25) = 0.3;

v0 = zeros(size(x));             %zero potential

[energy,phi,H]=schrod(len,npoints,v,num_sol);		    %call Schrod.
[energy0,phi0,H0]=schrod(len,npoints,v0,num_sol);		%call Schrod.

% some basic operators: x-operator and k-operator
xop = diag(x);
kop = (diag(ones(npoints,1),1)-diag(ones(npoints,1),-1))/(2i*dx);

%setup the initial state and normalize
x0 = 15; vel0 = 0.3;
psi = exp(-(x-x0).^2./(2*1^2)).*exp(1i*emass*vel0*x/hbar); psi = psi/sqrt(psi'*psi);
fprintf('energy of the state: %g eV\n', psi(2:end-1)'*H*psi(2:end-1));
fprintf('dispersion of the state squared: %g nm^2\n', real(psi'*xop^2*psi) - real(psi'*xop*psi)^2);

%decompose into eigenstates
c_old = phi'*psi;
c0_old = phi0'*psi;

fprintf('1 - sum |c(i)|^2 = %g (ideally 0)\n', 1 - sum(abs(c_old).^2));

figure(1)
loglog(energy, abs(c_old).^2, '.'); % decomposition of the packet
xlabel('energy of j(th)-mode (eV)'); ylabel('coefficient |c_j|^2');

t = 0:1:80; % time in fs
%t = [0 40 80];
for k = 1:length(t)
    % compute new state
    c = c_old .* exp(-1i*energy*t(k)/hbar);
    c0 = c0_old .* exp(-1i*energy0*t(k)/hbar);
    cmat = repmat(c, [1 npoints+1]).';
    cmat0 = repmat(c0, [1 npoints+1]).';
    psi_t = sum(cmat.*phi,2);
    psi_t0 = sum(cmat0.*phi0,2);
    
    % find current at each point
    ddx_psi_t = (1i*kop)*psi_t; % derivative of psi_t
    jx = real(1i*(psi_t.*conj(ddx_psi_t) - conj(psi_t).*ddx_psi_t));
    
    % compute expectation values
    xav = real(psi_t'*xop*psi_t);
    vav = hbar*real(psi_t'*kop*psi_t)/emass;
    Eav = real(psi_t(2:end-1)'*H*psi_t(2:end-1));
    KEav = real(psi_t(2:end-1)'*H0*psi_t(2:end-1)); % H0 is KE operator since V(x) = 0

    figure(2)
    subplot(2,1,1); % show probability density
    plot(x, abs(psi_t).^2, '-', x, abs(psi_t0).^2, '--', x, v*0.1); title(sprintf('t=%g{fs}, <x>=%.2g{nm}, <v>=%.2g{nm/fs}, <E>=%.2g{eV}, <KE>=%.2g{eV}', t(k), xav, vav, Eav, KEav));
    legend('\psi(x,t)','\psi_{free}(x,t)','potential');
    xlabel('distance (nm)'); ylabel('|\psi(x,t)|^2 (a.u.)');
    subplot(2,1,2); % show current
    plot(x, jx); xlabel('distance (nm)'); ylabel('j_x (a.u.)');
    drawnow;
    %pause();
end

figure(3) % comparison between time operator and decomposition method
U = expm(-1i*H*t(k)/hbar); % time evolution operator
plot(x(2:end-1), abs(U*psi(2:end-1)).^2, '.', x, abs(psi_t).^2);
xlabel('distance (nm)'); ylabel('|\psi(x,t)|^2 (a.u.)');
legend('evolution operator', 'decomposition');
title(sprintf('t=%g{fs}', t(k)));
