function [Trans,E] = mytunnel(volt,Emax)
N=201;                              %number of samples of potential
x = linspace(0, 40E-9, N);          %setup positions
dL = x(2)-x(1);                     %position increment (m)

V = zeros(size(x));                 %potential initialized to zeros

%setup the potential
V(x >= 16E-9 & x <= 17.5E-9) = 0.3;     %barrier of 0.4nm width
V(x >= 22.5E-9 & x <= 24E-9) = 0.3;     %barrier of 0.2nm width

Vtilt = zeros(size(x));
pos = x >= 5E-9 & x <= 35E-9;
Vtilt(pos) = -volt*(x(pos) - 5E-9)/30E-9;
Vtilt(x > 35E-9) = -volt;

V = V + Vtilt;

if nargin < 2
    Emax=1;                             %maximum particle energy (eV)
end

Emin=1E-10;                         %some small number to avoid 1/0
npoints=500;                        %number of points in energy plot
dE=Emax/npoints;                    %energy increment (eV)
hbar=1.05457159e-34;                %Planck's constant (J*s)
m0=9.109382e-31;                    %bare electron mass (kg)
meff=0.07;                             %effective electron mass / m0
m=meff*m0;                          %effective electron mass (kg)
echarge=1.6021764e-19;              %electron charge (C)

for j=1:npoints
    E(j)=dE*j+Emin;
    bigT=[1,0;0,1];  %default value of matrix bigT
    for i=1:N
        k(i)=sqrt(2*echarge*m*(E(j)-V(i)))/hbar;  %wave vector at energy E
    end
    for n=1:(N-1)                          %multiply out propagation matrix
        alpha = exp(1i*k(n)*dL);
        beta = k(n)/k(n+1);
        t(1,1) = alpha*(1+beta);
        t(1,2) = 1/alpha*(1-beta);
        t(2,1) = alpha*(1-beta);
        t(2,2) = 1/alpha*(1+beta);
        t = t*0.5;   %don't forget a factor of 1/2 for each element
        bigT=t*bigT;
    end
    Trans(j)=1-abs(bigT(2,1)/bigT(2,2)).^2;         %transmission coefficient
end

figure(1)
% show potential
subplot(1,2,1); plot(x*1E9,V); xlabel('Position, x (nm)'),ylabel('Potential energy, V (eV)');
axis([-Inf, Inf, -0.5, 1.4]);
% show the actual transmission coefficient
subplot(1,2,2);
plot(Trans,E, 'r-');
xlabel('Transmission coefficient'),ylabel('Energy, E (eV)');
axis([-Inf, Inf, -0.5, 1.4]);
end