function energy = twoatom(a, b)
%numerical solution to Schroedinger equation for
%rectangular potential well with infinite barrier energy

length = 3;	                     %length of well (nm)
npoints=1200;					 %number of sample points
x=linspace(0,length,npoints+1);	 %position of sample points
num_sol=6;			             %number of solutions sought

v = 10*ones(size(x));              %potential (eV)

p1 = x >= length/2 - b/2 - a & x <= length/2 - b/2;
p2 = x <= length/2 + b/2 + a & x >= length/2 + b/2;

v(p1 | p2) = 0;

[energy,phi]=schrod(length,npoints,v,num_sol);		%call Schrod.

for i=1:num_sol
    fprintf('eigenenergy (%d) = %f eV\n', i, energy(i));
end

figure(1);
plot(x,v,'b');xlabel('Distance (nm)'),ylabel('Potential energy, (eV)');
hold on;
plot(x, phi(:,[1:num_sol])*10+repmat(energy,[1 npoints + 1])');
hold off;

xlabel('Distance (nm)'),ylabel('Wave function');
hold off;